/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2020 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_main.c
* Version      : CodeGenerator for RL78/G13 V2.05.05.01 [25 Nov 2020]
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : This file implements main function.
* Creation Date: 2021/04/15
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_port.h"
#include "r_cg_serial.h"
#include "r_cg_timer.h"
#include "r_cg_wdt.h"
#include "r_cg_it.h"
/* Start user code for include. Do not edit comment generated here */
#include "r_iic_lib.h"
//#include "r_timer_user.h"
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
#pragma interrupt r_it_interrupt(vect=INTIT,bank=RB2)
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
#define     MAX_DATA    ( 16 )                  /* max data number for one time */
#define     SLAVE_ADDR  ( 0xA2 )                /* IIC bus slave address        */
#define     RAM_TOP     ( 0x00 )                /* OtRTC̐擪AhX		*/
#define     LED_ON      ( 0 )                   /* LED on data                  */
#define     RETRY       ( 10 )                  /* error retry times            */

volatile uint8_t    g_read_ram[16];          	/* receive RAM data buffer      */
volatile uint8_t    g_write_data[MAX_DATA+1];   /* transmit data buffer         */


static uint8_t const __far  RAM_COMMAND[4] =
{
    0b00000000,                         /* RTC@eXgWX^Zbg  */
    0b00000000,                         /*iݒR}hQj		*/
    0b00000000,                         /*iݒR}hRj      	*/
    0b00000000                          /*iݒR}hSj  		*/
};

//@I2C@ǂݏoobt@
static uint8_t const __far  RAM_DATA[2][16];

volatile uint8_t   g_1ms_status;        /* count end flag (1ms unit)    */
volatile uint8_t   g_1ms_count;         /* 1ms counter                  */


void wait_ms(uint8_t);                 	/* wait milli second            */
void wait_SW(void);                   	/* wait for SW is pushed        */

uint8_t r_IIC_put_data( uint8_t s_addr, uint8_t r_addr, uint8_t __far * const buffer, uint8_t tx_num );
uint8_t r_IIC_get_data( uint8_t s_addr, uint8_t r_addr, uint8_t * const buffer, uint8_t rx_num );



//I2C@ǂݏoobt@()
//unsigned char tx_data[16];
//UART@tO
volatile uint8_t g_tx_ready_flag;       /* M\tO       */
//UART@obt@
volatile uint8_t static g_tx_buff[16];  /* Mf[^Eobt@ */


void delay_ms(int ms);

/* End user code. Do not edit comment generated here */
void R_MAIN_UserInit(void);

/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void main(void)
{
    R_MAIN_UserInit();
    /* Start user code. Do not edit comment generated here */
	
		
  {

    uint8_t status;
    uint8_t __far const *pt_txdata;             /* write data to slave pointer  */
    uint8_t *pt_ram;                            /* read RAM data pointer        */
    uint8_t reg_addr;                           /* register address in slave    */
    uint8_t temp_pt;                            /* work pointer for read data   */
    uint8_t temp_data;                          /* work area for read data      */
	
	/* UARTeXgf[^*/
	/*
	tx_data[0] = 'A';
	tx_data[1] = 'B';
	tx_data[2] = 'C';
	tx_data[3] = '\n';
	tx_data[4] = '\r';
 	*/
	


    reg_addr = 0x00;                            /* RTC擪AhX@0x00 */

/*--------------------------------------------------------------*/
/*      RTC@eXgWX^Zbg								*/
/*--------------------------------------------------------------*/

    status = r_IIC_put_data( SLAVE_ADDR,        /* select serial RAM            */
                             reg_addr,          /* index command register       */
                             (uint8_t __far *)&RAM_COMMAND[0], /* command       */
                             1 );               /* data number(command only)    */

    while ( status )
    {
        P7_bit.no0 = LED_ON;                    /* turn on LED when NACK        */
        HALT();                                 /* program halt                 */
    }

/*--------------------------------------------------------------*/
/*      ݒR}hQiłRTCeXgWX^Zbgj */
/*--------------------------------------------------------------*/

    status = r_IIC_put_data( SLAVE_ADDR,        /* select serial RAM            */
                             reg_addr,          /* index command register       */
                             (uint8_t __far *)&RAM_COMMAND[0], /* command       */
                             1 );               /* data number(command only)    */

    while ( status )
    {
        P7_bit.no0 = LED_ON;                    /* turn on LED when NACK        */
        HALT();                                 /* program halt                 */
    }


/*==============================================================*/
/*      RAM check main operation loop                           */
/*==============================================================*/

    while (1U)
    {

/*--------------------------------------------------------------*/
/*      Data read from serial RAM by 16 bytes unit              */
/*--------------------------------------------------------------*/
		pt_ram = (uint8_t *)&g_read_ram[0];  /* set pointer for read data*/
        status = r_IIC_get_data( SLAVE_ADDR,/* select serial RAM            */
                                RAM_TOP,   /* index serial RAM srea        */
                                pt_ram,     /* read data buffer address     */
                                0x10 );     /* read data number             */

        if ( status )
        {                                   /* read error ( not slave)      */
            P7_bit.no0 = LED_ON;            /* turn on LED when NACK        */
        }
        else
        {
            /* do nothing */
        }

		
		R_UART0_Send( (uint8_t *)g_read_ram, sizeof(g_read_ram) );    /* f[^MJn */
		g_tx_ready_flag = 0x00;             /* MtONA   */
		
		delay_ms(30);						/*@̔Ȃ҂Ԃݒ*/
		
		while( 0x00 == g_tx_ready_flag )    /* M҂             */
        {
            P7_bit.no1 = LED_ON;            /* UARTM@LED1@ON     */
        }			


/*--------------------------------------------------------------*/
/*      write data to serial RAM by 64 bytes unit               */
/*--------------------------------------------------------------*/
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before implementing main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_MAIN_UserInit(void)
{
    /* Start user code. Do not edit comment generated here */
	
	/* start IT                       */

    ITMK = INT_ENABLE;                  /* enable IT interrupt	    */
	R_UART0_Start();
    EI();

    wait_ms(250);                       /* wait for 250ms	        */

    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/*******************************************************************************

    IIC bus device control routines based on r_IIC_LIB.c

*******************************************************************************/

/*******************************************************************************
* Function Name: r_IIC_put_data
* Description  : This function transmit data to specified slave
* Arguments    : slave address, register address, data buffer address, data numer
* Return Value : 00:successs
*                not 00:error
********************************************************************************/
uint8_t r_IIC_put_data( uint8_t s_addr, uint8_t r_addr, uint8_t __far * const buffer, uint8_t tx_num )
{
    uint8_t cnt;
    uint8_t cnt2;
    uint8_t __far * ptr;                /* input data pointer           */
    uint8_t status;
/*  0x00 : success(IIC_SUCCESS)
    0x01 : transmit operating(IIC_USING)
    0x84 : no ACK from slave for data(NO_ACK)
    0x88 : no ACK for slave address(NO_SLAVE)
    0x8C : I2C BUS busy(BUS_BUSY)
*/
    uint8_t retry_count;                /* error retry counter          */

/*--------------------------------------------------------------*/
/*      copy communication parameters                           */
/*--------------------------------------------------------------*/

    cnt = tx_num;                       /* set transmit data number     */
    g_write_data[0] = r_addr;           /* set register address         */
    ptr = buffer;                       /* set data pointer             */

    for ( cnt2 = 0 ; cnt2 < cnt ; cnt2++ )
    {

/*--------------------------------------------------------------*/
/*      copy transmit data to transmit buffer                   */
/*--------------------------------------------------------------*/

        g_write_data[cnt2+1] = *ptr;    /* copy transmit data           */
        ptr++;                          /* move data pointer            */
    }

/*--------------------------------------------------------------*/
/*      start transmission                                      */
/*--------------------------------------------------------------*/

    for ( retry_count = RETRY ; retry_count > 0 ; )
    {

        status = R_IIC_Master_Send(s_addr, (uint8_t *__near)g_write_data, cnt+1);

        if( IIC_SUCCESS == status )     /* check start successfully     */
        {

/*--------------------------------------------------------------*/
/*      transmit start and wait for transmission end            */
/*--------------------------------------------------------------*/

            do
            {                           /* wait for transmission end    */

                status = R_IIC_check_comstate(); /* get IIC status      */

            }while( IIC_USING == ( status & STS_MASK ) );
        
        }
        else
        {
            /* do nothing */
        }

/*--------------------------------------------------------------*/
/*      transmission end and check result of communication      */
/*--------------------------------------------------------------*/

        if( NO_SLAVE == status )
        {                               /* no ACK for slave address     */
            R_IIC_StopCondition();      /* issue stop condition         */
            delay_us(10);               /* wait for retry timing        */
            retry_count--;
        }
        else
        {
            retry_count = 0;
        }
    }
/*
    release IIC  bus
*/
    if( NO_SLAVE != status )
    {
        R_IIC_StopCondition();          /* issue stop condition         */
    }
    else
    {
            /* do nothing */
    }

    return (status);

}

/*******************************************************************************
* Function Name: r_IIC_get_data
* Description  : This function transmit data to specified slave
* Arguments    : slave address, register address, data buffer address, data numer
* Return Value : 00:successs
*                not 00:error
********************************************************************************/
uint8_t r_IIC_get_data( uint8_t s_addr, uint8_t r_addr, uint8_t * const buffer, uint8_t rx_num )
{

    uint8_t status;
    uint8_t retry_count;                /* error retry counter          */

    g_write_data[0] = r_addr;           /* set register address         */

/*==============================================================*/
/*      start i2c communication                                 */
/*==============================================================*/

    for ( retry_count = RETRY ; retry_count > 0 ; )
    {

/*--------------------------------------------------------------*/
/*      transmit register address                               */
/*--------------------------------------------------------------*/

        status = R_IIC_Master_Send(s_addr, (uint8_t *__near)g_write_data, 1);

        if( IIC_SUCCESS == status )     /* check start successfully     */
        {

/*--------------------------------------------------------------*/
/*      transmission start and wait for transmission end        */
/*--------------------------------------------------------------*/

            do
            {

                status = R_IIC_check_comstate(); /* get IIC status      */

            }while( IIC_USING == ( status & STS_MASK ) );

        }
        else
        {
                /* do nothing */
        }

/*--------------------------------------------------------------*/
/*      transmission end and check result                       */
/*--------------------------------------------------------------*/

        if( NO_SLAVE == status )
        {                               /* no ACK for slave address     */
            R_IIC_StopCondition();      /* issue stop condition         */
            delay_us(10);               /* wait for retry timing        */
            retry_count--;
        }
        else
        {
            break;
        }
    }

/*--------------------------------------------------------------*/
/*      transmission end                                        */
/*--------------------------------------------------------------*/

    if ( IIC_SUCCESS == status )        /* check result         */
    {

/*--------------------------------------------------------------*/
/*      start IIC receiving                                     */
/*--------------------------------------------------------------*/

        status = R_IIC_Master_Receive(s_addr, (uint8_t *__near)buffer, rx_num);

        if( IIC_SUCCESS == status )     /* check start successfully     */
        {

/*--------------------------------------------------------------*/
/*      wait for receive end                                    */
/*--------------------------------------------------------------*/

            do
            {
                status = R_IIC_check_comstate();

            }while( IIC_USING == ( status & STS_MASK ) );
        }
        else
        {
            /* do nothing */
        }

    }
    else
    {
        /* do nothing */
    }

    if( NO_SLAVE != status )
    {
        R_IIC_StopCondition();          /* issue stop condition     */
    }
    else
    {
            /* do nothing */
    }

    return (status);

}
/***********************************************************************************************************************

    Long range timer functions

***********************************************************************************************************************/

/*******************************************************************************
* Function Name: wait_ms
* Description  : This function wait for upto 255ms
* Arguments    : ms
* Return Value : none
********************************************************************************/
void wait_ms(uint8_t wait_time)
{
    g_1ms_status = FALSE;               /* clear timeup flag            */
    g_1ms_count = wait_time;            /* set remain time              */
    ITIF = FALSE;                       /* clear interrupt flag	        */

    ITMC |= 0x8000;                     /* start 12bit IT               */
/*--------------------------------------------------------------*/
/*      wait for remaining time                                 */
/*--------------------------------------------------------------*/

    while ( FALSE == g_1ms_status )     /* wait for IT interrupt        */
    {
        HALT();
        NOP();
    }

    ITMC &= 0x0FFF;                     /* stop 12bit IT operation      */
}

/********************************************************************************
* Function Name: r_it_interrupt
* Description  : This function is 1milli sec interrupt service routine.
* Arguments    : None
* Return Value : None
*********************************************************************************/
static void r_it_interrupt(void)
{

    if (( g_1ms_count > 0 ) && ( g_1ms_status == FALSE ))
    {   /* if remaining time and not timeup them count down */
        g_1ms_count--;
        if ( g_1ms_count == 0 )         /* check remaining time         */
        {
            g_1ms_status = TRUTH;       /* clear count down flag        */
        }
    }

}

/********************************************************************************
* Function Name: UART MC^[o^C}
* Description  : h~pC^[o^C}
* Arguments    : None
* Return Value : None
*********************************************************************************/

void delay_ms(int ms)
{
  volatile int i,j;
  for(i=0;i<ms;i++)
  {
    for(j=0;j<2963;j++)
    {

    }
  }
}

/* End user code. Do not edit comment generated here */
